Attribute VB_Name = "Module1"
'############################################################################
'# Program Name:  BatchPPU - Batch Physical Properties Update               #
'# Programmer: Jason Newell, The Charles Machine Works, Inc.                #
'# Description: Program recursively traverses through assembly structure    #
'#              and prompts user to update Physical Properties of Part and  #
'#              Sheet Metal Documents that are not already updated.         #
'############################################################################
Option Explicit

Sub Main()
    On Error Resume Next
    
    Dim objSeApp As SolidEdgeFramework.Application
    Dim objAssyDoc As SolidEdgeAssembly.AssemblyDocument
    
    
    Set objSeApp = GetObject(, "SolidEdge.Application") 'Get Edge Application Object
    
    If Not (objSeApp Is Nothing) Then
        If objSeApp.ActiveDocumentType = igAssemblyDocument Then    'Ensure we're working with an assembly.
            Set objAssyDoc = objSeApp.ActiveDocument
            If Not (objAssyDoc Is Nothing) Then
                If objAssyDoc.Occurrences.Count <> 0 Then   'Ensure we have Occurrences in the assembly.
                    ProcessAssembly objAssyDoc
                Else
                    MsgBox "Assembly does not have any occurrences.  Unable to continue.", vbCritical, "Error"
                End If
            Else
                MsgBox Err.Description, vbCritical, "Error"
            End If
        Else
            MsgBox "Please open an Assembly document before beginning.", vbCritical, "Error"
        End If
    Else
        MsgBox "Solid Edge is not current running.  Unable to continue.", vbCritical, "Error"
    End If

    MsgBox "BatchPPU is complete.", vbInformation
End Sub

Public Function ProcessAssembly(Assembly As SolidEdgeAssembly.AssemblyDocument)
    Dim objOccurrence As SolidEdgeAssembly.Occurrence
    Dim strPartNumber As String
    Dim i As Integer

    For i = 1 To Assembly.Occurrences.Count
        Set objOccurrence = Assembly.Occurrences(i)
        
        Select Case objOccurrence.OccurrenceDocument.Type
            Case igPartDocument, igSheetMetalDocument
                UpdatePhysicalProperties objOccurrence.OccurrenceDocument
        End Select
        
        If objOccurrence.Subassembly Then ProcessSubAssemblyOccurrence objOccurrence
    Next i
End Function

Public Function ProcessSubAssemblyOccurrence(SubAssemblyOccurrence As SolidEdgeAssembly.Occurrence)
    Dim i As Integer
    
    On Error Resume Next
    
    For i = 1 To SubAssemblyOccurrence.SubOccurrences.Count
        Select Case SubAssemblyOccurrence.SubOccurrences.Item(i).SubOccurrenceDocument.Type
            Case igPartDocument, igSheetMetalDocument
                UpdatePhysicalProperties SubAssemblyOccurrence.SubOccurrences.Item(i).SubOccurrenceDocument
        End Select
        
        If SubAssemblyOccurrence.SubOccurrences.Item(i).Subassembly Then ProcessSubAssemblySubOccurrence SubAssemblyOccurrence.SubOccurrences.Item(i)
    Next i
End Function

Public Function ProcessSubAssemblySubOccurrence(SubAssemblySubOccurrence As SolidEdgeAssembly.SubOccurrence)
    Dim i As Integer
    Dim objSubOccurrence As SolidEdgeAssembly.SubOccurrence
    
    On Error Resume Next
    
    For i = 1 To SubAssemblySubOccurrence.SubOccurrences.Count
        Set objSubOccurrence = SubAssemblySubOccurrence.SubOccurrences(i)
        
        Select Case objSubOccurrence.SubOccurrenceDocument.Type
            Case igPartDocument, igSheetMetalDocument
                UpdatePhysicalProperties objSubOccurrence.SubOccurrenceDocument
        End Select
        
        If objSubOccurrence.Subassembly Then ProcessSubAssemblyOccurrence (objSubOccurrence)
    Next i
End Function

Public Function UpdatePhysicalProperties(objDocument As Object) As Boolean
    On Error Resume Next
    Dim objModel As Model
    Dim dblDensity As Double
    Dim dblAccuracyIn As Double
    Dim dblAccuracyOut As Double
    Dim dblVolume As Double
    Dim dblArea As Double
    Dim dblMass As Double
    Dim dblCofGravity() As Double
    Dim dblCofVolume() As Double
    Dim dblGlobalMoments() As Double
    Dim dblPrincipalMoments() As Double
    Dim dblPrincipalAxes() As Double
    Dim dblRadiiOfGyration() As Double
    Dim lngStatus As Long
    Dim dblStartRadiusArray(1 To 4) As Double, dblEndRadiusArray(1 To 4) As Double

    If Not PhysicalPropertiesAreValid(objDocument) Then
        If Not UserDefinedPhysicalPropertiesAreValid(objDocument) Then
            'Get the Model object in the active document.
            Set objModel = objDocument.Models(1)
        
            dblDensity = CDbl(InputBox("Please enter a density value for " & objDocument.Name, "Physical Properties"))
            
            If dblDensity <> 0 Then
                'Convert the English value to Metric
                dblDensity = dblDensity * 27.68 * 10 ^ 3
                'Update the Physical Properties with the new Density value.
                Call objModel.ComputePhysicalProperties(dblDensity, dblAccuracyIn, dblVolume, dblArea, dblMass, dblCofGravity, dblCofVolume, dblGlobalMoments, dblPrincipalMoments, dblPrincipalAxes, dblRadiiOfGyration, dblAccuracyOut, lngStatus)
                UpdatePhysicalProperties = True
            Else
                'For whatever reason, the Density value is not a valid value.
                Err.Raise vbObjectError, , "Please enter a positive numeric Density value."
                GoTo ErrorHandler
            End If
        Else
            MsgBox "User Defined Phsical Properties are already updated for file " & objDocument.Name, vbInformation, "Information"
        End If
    Else
        MsgBox "Phsical Properties are already updated for file " & objDocument.Name, vbInformation, "Information"
    End If
    Exit Function
    
ErrorHandler:
    MsgBox Err.Description, vbCritical, App.Title & " Error"

End Function

Public Function PhysicalPropertiesAreValid(theDocument As Object) As Boolean
    On Error Resume Next
    Dim dblDensity As Double
    Dim dblAccuracyIn As Double
    Dim dblAccuracyOut As Double
    Dim dblVolume As Double
    Dim dblArea As Double
    Dim dblMass As Double
    Dim dblCofGravity() As Double
    Dim dblCofVolume() As Double
    Dim dblGlobalMoments() As Double
    Dim dblPrincipalMoments() As Double
    Dim dblPrincipalAxes() As Double
    Dim dblRadiiOfGyration() As Double
    Dim lngStatus As Long
    Dim dblStartRadiusArray(1 To 4) As Double, dblEndRadiusArray(1 To 4) As Double
    Dim objModel As Model
    
    Set objModel = theDocument.Models(1)
    If Not (objModel Is Nothing) Then
        Call objModel.GetPhysicalProperties(lngStatus, dblDensity, dblAccuracyIn, dblVolume, dblArea, dblMass, dblCofGravity, dblCofVolume, dblGlobalMoments, dblPrincipalMoments, dblPrincipalAxes, dblRadiiOfGyration, dblAccuracyOut)
        If dblDensity <> 0 Then
            'PhysicalPropertiesAreValid = lngStatus
            PhysicalPropertiesAreValid = True
        End If
    End If
    
End Function

Public Function UserDefinedPhysicalPropertiesAreValid(theDocument As Object) As Boolean
    On Error GoTo ErrorHandler
    Dim dblVolume As Double
    Dim dblArea As Double
    Dim dblMass As Double
    Dim dblCofGravity() As Double
    Dim dblCofVolume() As Double
    Dim dblGlobalMoments() As Double
    Dim dblPrincipalMoments() As Double
    Dim dblPrincipalAxes() As Double
    Dim dblRadiiOfGyration() As Double
    Dim lngStatus As Long
    Dim dblStartRadiusArray(1 To 4) As Double, dblEndRadiusArray(1 To 4) As Double
    Dim objModel As Model
    
    'We only want to check if there is Geometry in the file.
    Set objModel = theDocument.Models(1)
    If Not (objModel Is Nothing) Then
        UserDefinedPhysicalPropertiesAreValid = True
        Call theDocument.GetUserPhysicalProperties(dblVolume, dblArea, dblMass, dblCofGravity, dblCofVolume, dblGlobalMoments, dblPrincipalMoments, dblPrincipalAxes, dblRadiiOfGyration)
    End If
    
    Exit Function
    
ErrorHandler:
    UserDefinedPhysicalPropertiesAreValid = False
End Function

